/*
 * This file is part of family-shop-mgr.
 *
 * family-shop-mgr is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * family-shop-mgr is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with family-shop-mgr.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author: Unai IRIGOYEN
 * Date: 12/07/2009
 *
 */

#include "FamilyShoppingManagerMainWindow.h"
#include <QMenuBar>
#include <QMessageBox>
#include <QInputDialog>

#include "ListManagerView.h"
#include "GoShoppingView.h"
#include "ShoppingTreeModel.h"

/*******************************************************************/
FamilyShoppingManagerMainWindow::FamilyShoppingManagerMainWindow(QWidget *parent)
    : QMainWindow(parent), activityView(NULL), //editMenu(NULL),
    showCheckedItemsAction(NULL), goShoppingAction(NULL), endShoppingAction(NULL)
{
    aboutAction = new QAction(tr("&About"), this);
    connect(aboutAction, SIGNAL(triggered()), this, SLOT(showAbout()));
    menuBar()->addAction(aboutAction);


    showListManager();
}

/*******************************************************************/
FamilyShoppingManagerMainWindow::~FamilyShoppingManagerMainWindow()
{
    delete activityView;
}

/*******************************************************************/
void FamilyShoppingManagerMainWindow::showListManager()
{
    menuBar()->clear();

    delete showCheckedItemsAction;
    delete endShoppingAction;

    delete activityView;
    activityView = new ListManagerView("ShoppingList.xml", this);
    setCentralWidget(activityView);

//    editMenu = new QMenu(tr("&Edit"), this);
    addCategoryAction = new QAction(tr("Add category"), this);
    connect(addCategoryAction, SIGNAL(triggered()),
            this, SLOT(addCategory()));
//    editMenu->addAction(addCategoryAction);
//    removeCategoryAction = new QAction(tr("Remove category"), this);
//    connect(removeCategoryAction, SIGNAL(triggered()),
//            this, SLOT(removeRow()));
//    editMenu->addAction(removeCategoryAction);
//    addItemAction = new QAction(tr("Add item"), this);
//    connect(addItemAction, SIGNAL(triggered()),
//            this, SLOT(insertRow()));
//    editMenu->addAction(addItemAction);
//    removeItemAction = new QAction(tr("Remove item"), this);
//    connect(removeItemAction, SIGNAL(triggered()),
//            this, SLOT(removeRow()));
//    editMenu->addAction(removeItemAction);
    menuBar()->addAction(addCategoryAction);

    goShoppingAction = new QAction(tr("Go shopping!"), this);
    connect(goShoppingAction, SIGNAL(triggered()),
            this, SLOT(showGoShopping()));
    menuBar()->addAction(goShoppingAction);

    menuBar()->addAction(aboutAction);
    update();
}

/*******************************************************************/
void FamilyShoppingManagerMainWindow::showGoShopping()
{
    menuBar()->clear();
//    delete editMenu;
    delete goShoppingAction;

    delete activityView;
    activityView = new GoShoppingView("ShoppingList.xml", this);
    setCentralWidget(activityView);

    showCheckedItemsAction = new QAction(tr("&Show checked"), this);
    showCheckedItemsAction->setCheckable(true);
    connect(showCheckedItemsAction, SIGNAL(toggled(bool)),
            activityView, SLOT(showChecked(bool)));
    menuBar()->addAction(showCheckedItemsAction);

    endShoppingAction = new QAction(tr("&End shopping"), this);
    connect(endShoppingAction, SIGNAL(triggered()),
            this, SLOT(showListManager()));
    menuBar()->addAction(endShoppingAction);

    menuBar()->addAction(aboutAction);
    update();
}

/*******************************************************************/
void FamilyShoppingManagerMainWindow::showAbout()
{
    QString text;
    text = "Application name: Family shopping manager\n";
    text += "Author: Unai IRIGOYEN\n\n";
    text += "Licence: GPL";
    QMessageBox::about(this,tr("About"), text);
}

/*******************************************************************/
/*
void FamilyShoppingManagerMainWindow::insertChild()
{
     QModelIndex index = ((ListManagerView*) activityView)->
                         selectionModel()->currentIndex();
     QAbstractItemModel *model = ((ListManagerView*) activityView)->model();

     if (model->columnCount(index) == 0) {
         if (!model->insertColumn(0, index))
             return;
     }

     if (!model->insertRow(0, index))
         return;

     for (int column = 0; column < model->columnCount(index); ++column)
     {
         QModelIndex child = model->index(0, column, index);
         model->setData(child, QVariant("[No data]"), Qt::EditRole);
         if (!model->headerData(column, Qt::Horizontal).isValid())
             model->setHeaderData(column, Qt::Horizontal,
                                  QVariant("[No header]"), Qt::EditRole);
     }

     ((ListManagerView*) activityView)->selectionModel()->
             setCurrentIndex(model->index(0, 0, index),
                             QItemSelectionModel::ClearAndSelect);
     ((ListManagerView*) activityView)->updateActions();
 }
 */

/*******************************************************************/
void FamilyShoppingManagerMainWindow::addCategory()
{
    ShoppingTreeModel *model = (ShoppingTreeModel*)
                               ((ListManagerView*) activityView)->model();

    bool* ok = new bool(false);
    QString name = QInputDialog::getText(this, tr("Enter category name"),
                                         tr("Category name:"),
                                         QLineEdit::Normal, QString(), ok);
    if(!ok)
        return;

    if(model->addCategory(name))
        ((ListManagerView*) activityView)->updateActions();
    delete ok;
}

/*******************************************************************/
void FamilyShoppingManagerMainWindow::addSubCategory()
{
    QModelIndex index = ((ListManagerView*) activityView)->
                        selectionModel()->currentIndex().parent();
    ShoppingTreeModel *model = (ShoppingTreeModel*)
                               ((ListManagerView*) activityView)->model();

    bool* ok = new bool(false);
    QString name = QInputDialog::getText(this, tr("Enter category name"),
                                         tr("Category name:"),
                                         QLineEdit::Normal, QString(), ok);
    if(!ok)
        return;

    if(model->addSubCategory(name, index.row()+1, index))
        ((ListManagerView*) activityView)->updateActions();

    delete ok;
}

/*******************************************************************/
void FamilyShoppingManagerMainWindow::addItem()
{
    QModelIndex index = ((ListManagerView*) activityView)->
                        selectionModel()->currentIndex().parent();
    ShoppingTreeModel *model = (ShoppingTreeModel*)
                               ((ListManagerView*) activityView)->model();

    bool* ok = new bool(false);
    QString name = QInputDialog::getText(this, tr("Enter item name"),
                                         tr("Item name:"),
                                         QLineEdit::Normal, QString(), ok);
    if(!ok)
        return;

    if(model->addItem(name, index.row()+1, index))
        ((ListManagerView*) activityView)->updateActions();

    delete ok;
}

/*******************************************************************/
void FamilyShoppingManagerMainWindow::removeCategoryOrItem()
{
     QModelIndex index = ((ListManagerView*) activityView)->
                         selectionModel()->currentIndex();
     ShoppingTreeModel *model = (ShoppingTreeModel*)
                                ((ListManagerView*) activityView)->model();
     if (model->removeCategoryOrItem(index))
         ((ListManagerView*) activityView)->updateActions();
}
