/*
 * This file is part of family-shop-mgr.
 *
 * family-shop-mgr is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * family-shop-mgr is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with family-shop-mgr.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author: Unai IRIGOYEN
 * Date: 12/07/2009
 *
 */

#ifndef SHOPPINGTREEITEM_H
#define SHOPPINGTREEITEM_H

#include <QList>
#include <QVariant>
#include <QVector>
#include <QSemaphore>

class ShoppingTreeItem : public QObject
{
Q_OBJECT

public:

    enum ItemType
    {
        Category = 0,
        Item = 1,

        NotDefined = -1
    };

    ShoppingTreeItem(const QVector<QVariant> &data, ShoppingTreeItem *parent = 0);
    ~ShoppingTreeItem();

    ShoppingTreeItem *child(int number);
    int childCount() const;
    int columnCount() const;
    QVariant data(int column) const;
    bool insertChildren(int position, int count, int columns);
    bool insertColumns(int position, int columns);
    ShoppingTreeItem *parent();
    bool removeChildren(int position, int count);
    bool removeColumns(int position, int columns);
    int childNumber() const;
    bool setData(int column, const QVariant &value);
    bool setItemType(const ItemType type);
    ItemType getItemType() const;
    void waitItemTypeDefinition();

signals:
    void childInserted(ShoppingTreeItem *item);
    void dataChanged(int column);
    void childRemoved(ShoppingTreeItem *item);

private:
    QList<ShoppingTreeItem*> childItems;
    QVector<QVariant> itemData;
    ShoppingTreeItem *parentItem;

    ItemType m_itemType;
    QSemaphore m_typeSemaphore;
};

#endif // SHOPPINGTREEITEM_H
