/*
 * This file is part of family-shop-mgr.
 *
 * family-shop-mgr is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * family-shop-mgr is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with family-shop-mgr.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Author: Unai IRIGOYEN
 * Date: 12/07/2009
 *
 */

#ifndef SHOPPINGTREEMODEL_H
#define SHOPPINGTREEMODEL_H

#include <QAbstractItemModel>
#include <QDomDocument>
#include <QModelIndex>
#include <QVector>
#include <QVariant>
#include <QHash>

class ShoppingTreeItem;

class ShoppingTreeModel : public QAbstractItemModel
{
Q_OBJECT

public:
    ShoppingTreeModel(const QString &xmlFileName, QObject *parent = 0);
    ~ShoppingTreeModel();

    bool addCategory(QString name);
    bool addSubCategory(QString name, int position,
                        const QModelIndex &parent = QModelIndex());
    bool addItem(QString name, int position,
                 const QModelIndex &parent = QModelIndex());
    bool removeCategoryOrItem(const QModelIndex &index = QModelIndex());

    QVariant data(const QModelIndex &index, int role) const;
    QVariant headerData(int section, Qt::Orientation orientation,
                        int role = Qt::DisplayRole) const;
    QModelIndex index(int row, int column,
                      const QModelIndex &parent = QModelIndex()) const;
    QModelIndex parent(const QModelIndex &index) const;

    int rowCount(const QModelIndex &parent = QModelIndex()) const;
    int columnCount(const QModelIndex &parent = QModelIndex()) const;

    Qt::ItemFlags flags(const QModelIndex &index) const;
    bool setData(const QModelIndex &index, const QVariant &value,
                 int role = Qt::EditRole);
    bool setHeaderData(int section, Qt::Orientation orientation,
                       const QVariant &value, int role = Qt::EditRole);

private:
    bool insertColumns(int position, int columns,
                           const QModelIndex &parent = QModelIndex());
    bool removeColumns(int position, int columns,
                           const QModelIndex &parent = QModelIndex());
    bool insertRows(int position, int rows,
                    const QModelIndex &parent = QModelIndex());
    bool removeRows(int position, int rows,
                    const QModelIndex &parent = QModelIndex());

signals:
    void xmlParseError(QString error, int line, int column);
    void invalidDocument();

public slots:
    void registerInsertedChild(ShoppingTreeItem *item);
    void deleteRemovedChild(ShoppingTreeItem *item);

protected:
    void parseCategoryElement(const QDomElement &element,
                              ShoppingTreeItem *parentItem = 0);
    QVector<QVariant> getColumnsFromItemElement(const QDomElement &element);
    bool updateDomElement(ShoppingTreeItem *item, int column);
    void updateXmlFile() const;

private:
    ShoppingTreeItem *getItem(const QModelIndex &index) const;

    ShoppingTreeItem *rootItem;
    QString m_xmlFileName;
    QDomDocument m_document;
    QHash<ShoppingTreeItem *, QDomElement> m_domElementForItem;
};

#endif // SHOPPINGTREEMODEL_H
